#include "main.h"
#include "driver.h"


//-----------------------------------INTERRUPT SERVICE RUTINS---------
void interrupt
global_isr(void) 
{
//-------------------------------------rtcc interrupt------------------
	if(TMR0IF){//interrupt every 2 ms
       if(++key_scan_timer >= KEY_SCAN_TIME){
            key_scan_timer=0;
            key_scan_flag=1;
       }
       if(current_mode == LEDS){
          if(++leds_timer >= LEDS_SHIFT_TIME){//leds shift time
               leds_flag=1;
               leds_timer=0;
          }
       }
       else{
            leds_flag=0;
            leds_timer=0;
       }
       if(current_mode == A2D){//a2d display update time
          if(++a2d_timer >= A2D_DISPLAY_UPDATE){
               update_a2d_result_flag=1;
               a2d_timer=0;
          }
       }
       else{
            update_a2d_result_flag=1;
            a2d_timer=0;
       }
	   TMR0IF=0;
	   TMR0=RELOAD; // reload timer1 values
    }
//----------------------------------------a2d interrupt----------------
    if(ADIF){
       ADIF=0;
       a2d_result=ADRESH;
    }
//---------------------------------------rs232 interrupt---------------
    if(RCIF)
		rs232_value=RCREG;		 
}


void init_all(){
	//----------------------------------------------
    ADIF=0;//reset a2d interrupt flag
	//------initiolization of pwm--------------------
    CCP1CON=PWM_OFF;
	PR2=99;  // choose 10 khz
	T2CON=0x05;// turn on timer2 and choose prescaler 1 : 4
	duty_cycle=50; //50%
	CCPR1L=((int)PR2+1)*(int)duty_cycle/100; // choose duty cycle 50 %,8 bit resolution
	pr2_digit[2]=PR2%10;
	pr2_digit[1]=PR2/10;
    pr2_digit[1]=pr2_digit[1]%10;
    pr2_digit[0]=PR2/100;
    duty_digit[1]=duty_cycle%10;
	duty_digit[0]=duty_cycle/10;
	//------initiolization of timer0-----------------
	GIE=ON;      // enable global interrupts
	PEIE=ON;     // enable peripherial interrupts 
	TMR0IF=0;    // reset timer1 interrupt flag
	TMR0IE=ON;  // disable timer1 interrupt
	T0CS=OFF;  // choose internal clock  Fosc/4
    PSA=OFF;
	PS0=ON;  // choose 1 : 64 prescaler
	PS1=OFF;  
	PS2=ON;  
	TMR0=RELOAD; // load timer0 registers with value 131
    //-----------------init A2D-----------------------
	ADCON1 = 0x0e;//select A0 analog and all others digital		
    select_a2d_channel(potenziometer);
	//------initiolization of lcd--------------------
	TRISA = 0x21; //0010 0001
	PORTA=0xff;	  //reset leds
    TRISB = 0xff; // portb input
	TRISC2 = 0;   
	RC2 = 0;  //reset buzzer output
	TRISD = 0x00; // Make all Port D data lines output
	PORTD = 0;    // portd reset
    TRISE = 0x00; // lcd control lines
	//------initiolization of lcd--------------------
	lcd_init (); //lcd initiolization rutin
    lcd_clear();
	//----------------------------------------------
    kbd_init();  //keyboard initiolization rutin
	i2c_init();  //i2c protocol initiolization rutin
	write_d2a_config(0); //choose d2a normal mode
	write_d2a_data(0);   //reset d2a output voltage
    //----------------rs232 initiolization----------
	SPBRG =51;     	
	TXSTA = 0b00100100;
	RCSTA = 0b10010000;
	TRISC6=1;
	TRISC7=1;
	RCIF=0;
    //--------------------------------------------
	//--------------------------reset variables-----------------
	update_a2d_result_flag=0;
	leds_flag=0;
	current_mode=0;
	leds_timer=0;
	key_scan_flag=0;
	key_scan_timer=0;
    direction=LEFT;
    last_mode=1;
    last_next_button=0;
    last_enter_button=0;
    next_time=0;
	a2d_int=0;
	a2d_flt1=0;
	a2d_flt2=0;
	a2d_result=0;
    a2d_timer=0;
	rs232_value=0;
	left_state=0;
	right_state=0;
	press_result=0;
	new_press=0;
	buzzer_state=0;
	cursor_state=0;
	first_time=0;
	rs232_state=0;
	last_rs232_value=0;
	store_state=0; 
	read_state=0; 
	eeprom_state=0;
	address=0;
	update_display=0;
    for(i=0;i<3;++i){
		add_digit[i]=0;
		d2a_digit[i]=0;
	}
}

//--------------------------------MAIN PROGRAM----------------------------
main(void)
{
	init_all();
	do{//always do this while
		switch(current_mode){//check current mode value and execute relevant subrutin
			case LEDS:
				leds_rutin();
			break;
			case BUTTONS:
				buttons_rutin();
			break;
			case A2D:
				a2d_rutin();
			break;
			case D2A:
				d2a_rutin();
			break;
			case BUZZER:
				buzzer_rutin();
			break;
			case RS232:
				rs232_rutin();
			break;
			case USB:
				rs232_rutin();
			break;
			case EXT_EEPROM:
				ext_eeprom_rutin();
			break;
			case RF:
				rf_rutin();
			break;
		} 
		if(key_scan_flag){// scan the keyboard every 10ms
			press_result=key_scan();
			if(press_result != NULL)//update new_press value if press_result other than NULL
				new_press=press_result;		
			key_scan_flag=0;
		} 
	}while(TRUE);
}

