////                             KBD.C                                 ////
////                  Generic keypad scan driver                       ////
////                                                                   ////
////  kbd_init()   Must be called before any other function.           ////
////                                                                   ////
////  c = kbd_getc(c)  Will return a key value if pressed or /0 if not ////
////                   This function should be called frequently so as ////
////                   not to miss a key press.                        ////
///////////////////////////////////////////////////////////////////////////

////////////////// The following defines the keypad layout 



#define COL0 (1 << 2)   //B2
#define COL1 (1 << 3)   //B3
#define COL2 (1 << 4)   //B4
#define COL3 (1 << 5)   //B5

#define ROW0 (1 << 4)   //D4
#define ROW1 (1 << 5)   //D5
#define ROW2 (1 << 6)   //D6
#define ROW3 (1 << 7)   //D7
//
#define ALL_ROWS (ROW0|ROW1|ROW2|ROW3)
#define ALL_COLS (COL0|COL1|COL2|COL3)
#define KBD_COLS PORTB
#define KBD_ROWS PORTD
#define NUMBER_OF_COLS  4
#define NUMBER_OF_ROWS  4

// Keypad layout:
char const KEYS[4][4] = {{'1','2','3','A'},
                         {'4','5','6','B'},
                         {'7','8','9','C'},
                         {'*','0','#','D'}};

kbd_init (){
	RBPU=0;  // enable internal pullups  ;valid only in pcm family 
	TRISB|=ALL_COLS; 
	TRISD&=~ALL_ROWS;
}

unsigned char last_key=0;
unsigned char row,col,keychar;

unsigned char key_scan(){
	keychar=NULL;
	for(row=0 ; row < NUMBER_OF_COLS ; ++row){
		switch(row){
			case 0:
				KBD_ROWS=(KBD_ROWS | ALL_ROWS)& ~ROW0; 
			break;
			case 1: 
				KBD_ROWS=(KBD_ROWS | ALL_ROWS)& ~ROW1; 
			break;
			case 2: 
				KBD_ROWS=(KBD_ROWS | ALL_ROWS)& ~ROW2; 
			break;
			case 3: 
				KBD_ROWS=(KBD_ROWS | ALL_ROWS)& ~ROW3; 
			break;
		}
		for(col=0; col < NUMBER_OF_ROWS; ++col){
			switch (col){
				case 0: 
					if((KBD_COLS & COL0)==0) keychar=KEYS[row][col];
				break;
				case 1:
					if((KBD_COLS & COL1)==0) keychar=KEYS[row][col];
				break;
				case 2: 
					if((KBD_COLS & COL2)==0) keychar=KEYS[row][col];
				break;
				case 3:
					if((KBD_COLS & COL3)==0) keychar=KEYS[row][col]; 
				break;
			}
		} 
	}
	KBD_ROWS=(KBD_ROWS & ~ALL_ROWS);
	DelayUs(10);
	if((KBD_COLS & ALL_COLS) != ALL_COLS && last_key == keychar) 
		return NULL;
	else{ 
		last_key=keychar; 
		return (keychar);
	}
} 
 
