/*
 *	LCD interface example
 *	Uses routines from delay.c
 *	This code will interface to a standard LCD controller
 *	like the Hitachi HD44780. It uses it in 4 bit mode, with
 *	the hardware connected as follows (the standard 14 pin 
 *	LCD connector is used):
 *	
 *	
 *	To use these routines, set up the port I/O (TRISA, TRISD) then
 *	call lcd_init(), then other routines as required.
 *	
 */

#include	"lcd.h"
 

#define LCD_RS RE0
#define LCD_WR RE1
#define LCD_EN RE2

#define	LCD_STROBE	((LCD_EN = 1),(LCD_EN=0))
#define OUTPUT 0x00
#define INPUT  0xff

/* write a byte to the LCD in 4 bit mode */


void
lcd_iwrite(unsigned char c)
{
	LCD_RS = 0;
    PORTD = (PORTD & 0xF0) |  (c >> 4);
	LCD_STROBE;
	PORTD = (PORTD & 0xF0) |  (c & 0x0F);
	LCD_STROBE;
	DelayUs(40);
}



/*
 * 	Clear and home the LCD
 */

void
lcd_clear(void)
{
	lcd_iwrite(0x01);
	DelayMs(2);
}



/* write one character to the LCD */

void
lcd_putch(char c)
{
	LCD_RS = 1;	// write characters
	PORTD = (PORTD & 0xF0) |  (c >> 4);
	LCD_STROBE;
	PORTD = (PORTD & 0xF0) |  (c & 0x0F);
	LCD_STROBE;
	DelayUs(50);
}


/* write a string of chars to the LCD */

void
lcd_puts(const char * s)
{
	while(*s)
		lcd_putch(*s++);
}



unsigned char lcd_read ()
{
unsigned char value;
TRISD = INPUT;
LCD_RS = 1;
LCD_WR = 1;
//
LCD_STROBE;
value = (PORTD & 0x0F);
value=value<<4;
LCD_STROBE;
value = ((PORTD & 0x0F) |  value);
DelayUs(40);
TRISD = OUTPUT;
LCD_WR = 0;
return (value);
}


/*
 * Go to the specified position
 */

void
lcd_goto(unsigned char pos)
{
	lcd_iwrite(0x80+pos);
}
	
void
lcd_gotoxy(unsigned char x,unsigned char y)
{
	if (x==0)
   { 
    LCD_RS = 0;
	lcd_iwrite(0x80+y);
   }
    else if  (x==1)
   { 
    LCD_RS = 0;
	lcd_iwrite(0xC0+y);
   }

}


/* initialise the LCD - put into 4 bit mode */
void
lcd_init(void)
{
    LCD_WR = 0;
	LCD_RS = 0;	// write control bytes
	DelayMs(100);	// power on delay
	PORTD|=0x3;	// attention!
	LCD_STROBE;
	DelayMs(5);
	LCD_STROBE;
	DelayUs(100);
	LCD_STROBE;
	DelayMs(5);     
	PORTD|=0x2;	// return home
	LCD_STROBE;
	DelayUs(40);
	lcd_iwrite(0x28);	// 4 bit mode, 1/16 duty, 5x8 font, 2 lines
	lcd_iwrite(0x0c);	// display off
	lcd_iwrite(0x01);	// display on, blink curson off
	lcd_iwrite(0x06);	// entry mode
}

// clear 1 line and go to the start of the 1 line

void lcd_clear_1_line ()
{
lcd_gotoxy (0,0);
lcd_puts ("                ");
lcd_gotoxy (0,0);
}

// clear 2 line and go to the start of the 2 line

void lcd_clear_2_line ()
{
lcd_gotoxy (1,0);
lcd_puts ("                ");
lcd_gotoxy (1,0);
}

void lcd_dispnum ( unsigned char num )
{
char string[3];
unsigned char digits[3],count;

digits[2]=num%10;
num/=10;
digits[1]=num%10;
digits[0]=num/10;

for (count=0;count<3;++count)
   {
    switch (digits[count])
      {
       case 0 : string[count]='0'; break;
       case 1 : string[count]='1'; break;
       case 2 : string[count]='2'; break;
       case 3 : string[count]='3'; break;
       case 4 : string[count]='4'; break;
       case 5 : string[count]='5'; break;
       case 6 : string[count]='6'; break;
       case 7 : string[count]='7'; break;
       case 8 : string[count]='8'; break;
       case 9 : string[count]='9'; break;
       default: string[count]='p'; 
      }
   }
if (string[0] != '0') 
	for (count=0;count<3;++count) lcd_putch (string[count]);
else 
	for (count=1;count<3;++count) lcd_putch (string[count]);
}

